"""
This library is an implementation of Stakeholder Specific Vulnerability Categorization(SVCC)
"""
__version__ = "0.0.1"

from .helpers import initialize_db, get_db_conn, get_cisa_kevc
from .initialize import start_script



EXPOSURE = ["unlikely", "probable", "unavoidable"]
EXPLOITATION = ["none", "PoC", "active"]
Utility = ["laborious", "complex", "efficient"]
HUMAN_IMPACT = ["low", "medium", "high", "very high"]
PRIORITY = ["defer", "scheduled", "out-of-cycle", "immediate"]
PRODUCTION = "production"
NONPRODUCTION = "non_production"
DB = "db"
STORAGE = "storage"
COMPUTE = "compute"
VERYHIGH = "very high"
HIGH = "high"
MEDIUM = "medium"
LOW = "low"



import sys
import logging

from rapticoressvc import helpers
from rapticoressvc.ssvc_ore import ssvc_recommendations


def start_script(asset, vul_details, public_status, environment, asset_type, asset_criticality):
    engine_check = helpers.initialize()
    if not engine_check:
        logging.error('DB engine could not be Initialized')
        sys.exit(1)
    result = ssvc_recommendations(asset, vul_details, public_status, environment, asset_type, asset_criticality)
    return result


import os
import pandas as pd
import logging

absolute_path = os.path.dirname(__file__)

full_path = absolute_path

PATHS = {
    'deployer': os.path.join(full_path, "priority-options_v2.csv"),
    'impact': os.path.join(full_path, "ssvc_impact_options.csv"),
    'utility': os.path.join(full_path, "ssvc_utility_options.csv"),
}

DEFAULTS = {
    'deployer': {
        # An analyst should feel comfortable selecting none if they (or their search scripts) have performed searches
        # in the appropriate places for public PoCs and active exploitation (as described above) and found none.
        "Exploitation": "none",
        "Exposure": "unavoidable"
    },
    'impact': {
        "Environment": "none"
    },
    'utility': {
        "public_status": "none"
    }
}


def _load_csvs(path_dict):
    data = {}
    for key, path in path_dict.items():
        df = pd.read_csv(path)
        data[key] = df
    return data


DATA = _load_csvs(PATHS)

# confirm that PATHS and DATA keys match
assert (set(PATHS.keys()) == set(DATA.keys()))


def lookup(key, query_dict, use_defaults=True):
    # get the full table
    df = DATA[key]

    if use_defaults:
        # copy the defaults before we use them
        defaults = DEFAULTS.get(key, {})
        q = dict(defaults)
    else:
        q = {}

    q.update(query_dict)

    # with each pass, slice the table
    for k, v in q.items():
        df = df.loc[df[k] == v]
    return df


def outcome_dist(df, normalize=True):
    """
    Given a dataframe representing an SSVC tree fragment,
    compute and return the distribution of outcomes
    """
    return df['Priority'].value_counts(normalize=normalize)


def outcome_impact_dist(df, normalize=True):
    """
    Given a dataframe representing an SSVC tree fragment,
    compute and return the distribution of outcomes
    """
    return df['Impact'].value_counts(normalize=normalize)


def outcome_utility_dist(df, normalize=True):
    """
    Given a dataframe representing an SSVC tree fragment,
    compute and return the distribution of outcomes
    """
    return df['Utility'].value_counts(normalize=normalize)


def calculate_impact(query):
    for key, df in DATA.items():
        df = lookup('impact', query)
        recommendations = outcome_impact_dist(df).round(decimals=3).to_dict()
        recommendations = list(recommendations.keys())[0]
        return recommendations


def calculate_utility(query):
    for key, df in DATA.items():
        try:
            df = lookup('utility', query)
            recommendations = outcome_utility_dist(df).round(decimals=3).to_dict()
            recommendations = list(recommendations.keys())[0]
            return recommendations
        except Exception as e:
            logging.error(e)
            return "complex"


def calculate_recommendation(query):
    for key, df in DATA.items():
        df = lookup('deployer', query)
        recommendations = outcome_dist(df).round(decimals=3).to_dict()
        return recommendations


def main(query):
    recommendations = calculate_recommendation(query)
    return recommendations


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-

# Learn more: https://github.com/kennethreitz/setup.py

from setuptools import setup, find_packages

REQUIRED = []

with open('README.md') as f:
    readme = f.read()

with open('LICENSE') as f:
    license = f.read()

with open('requirements.txt') as f:
    while True:
        line = f.readline()
        if not line:
            break
        REQUIRED.append(line.strip())

setup(
    name='rapticoressvc',
    description='Rapticore SSVC ORE Miner',
    classifiers=[
        "Programming Language :: Python :: 3.10",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    long_description_content_type="text/markdown",
    long_description=readme,
    author='Rapticore, Inc',
    author_email='support@rapticore.com',
    url='',
    license=license,
    install_requires=REQUIRED,
    packages=find_packages(exclude=('tests', 'docs')),
    include_package_data=True,
    package_data={
            "rapticoressvc": ["ssvc_impact_options.csv", "priority-options_v2.csv", "ssvc_utility_options.csv", "data_vulnerability.csv"],
        },
)

